library(modreg)
convert.affy <- function(file,CDFfile=NULL,CELfiles=NULL,path="",read=T,chip.names=NULL,keep.controls=F){
  if(missing(file)) stop("Argument file missing, with no default\n")
  if(is.null(CDFfile) + is.null(CELfiles)==1) stop("Arguments CDFfile and CELfiles must be both missing or both specified\n")
  if(is.null(CDFfile) || is.null(CELfiles))
    system(paste(path,"cel2tab.pl ",file,sep=""))
  else
    system(paste(path,"cel2tab.pl ",CDFfile," ",paste(CELfiles,collapse=" ")," ",file,sep=""))
  if(read)
    return(read.affy(file,chip.names,keep.controls))
  else
    return(NULL)
}
    
read.affy <- function(file,chip.names=NULL,keep.controls=F){
  Data <- read.table(file,header=T,as.is=T)
  nchips <- dim(Data)[2] - 5
  if(is.null(chip.names)) chip.names <- names(Data)[-c(1:5)]
  else{
    if(length(chip.names)!=nchips){
      warning("Not the same number of chips than chip names. Assigning names from file.\n") 
      chip.names <- names(Data)[-c(1:5)]
    }
  }
  ncol <- max(Data$x)+1
  nrow <- max(Data$y)+1
  n <- dim(Data)[1]
  pmindex <- (1:n)[Data$pm.or.mm == 1]
  mmindex <- (1:n)[Data$pm.or.mm == 0]
  if(any((pmindex+ncol)-mmindex))
    warning("MMs are not underneath PMs\n")
  pm <- matrix(as.matrix(Data[pmindex,-c(1:5)]),ncol=nchips) ##contains all pms
  mm <- matrix(as.matrix(Data[mmindex,-c(1:5)]),ncol=nchips) ##contains all mms
  dimnames(pm)<- list(NULL,chip.names)
  dimnames(mm)<- list(NULL,chip.names)
  probe.names <- Data[pmindex,1]
  x <- Data$x[pmindex]
  y <- Data$y[pmindex]
  probe.numbers <- Data$number[pmindex]
  if(!keep.controls){
    return(list(pm=pm,mm=mm,probe.names=probe.names,
                probe.numbers=probe.numbers,
                x=x,y=y,ncol=ncol,nrow=nrow,nchips=nchips,
                nprob=length(pmindex),chip.names=chip.names))
  }
  else{
    control.index <- (1:n)[Data$pm.or.mm==-1]
    control.names <- Data[control.index,1]
    control.values <-matrix(as.matrix(Data[control.index,-c(1:5)]),ncol=nchips)
    dimnames(control.values) <- list(NULL,chip.names)
    control.x <- Data$x[control.index]
    control.y <- Data$y[control.index]
    control.probe.numbers <- Data$number[control.index]
    return(list(pm=pm,mm=mm,probe.names=probe.names,
                probe.numbers=probe.numbers,
                x=x,y=y,ncol=ncol,nrow=nrow,nchips=nchips,
                nprob=length(pmindex),chip.names=chip.names,
                control.names=control.names,control.values=control.values,
                control.x=control.x,control.y=control.y,
                control.probe.numbers=control.probe.numbers))
  }
}


#
# parts of code copied from code written by Cheng Li, Fred A. Wright, Ben Bolstad
#
### Implementation of reduced model of Li C, Wong WH PNAS 2001 98:31-36
fit.li.wong <- function(data.matrix, remove.outliers=T,
                        normal.array.quantile=0.5,
                        normal.resid.quantile=0.9,
                        large.threshold=3,
                        large.variation=0.8,
                        outlier.fraction=0.14,
                        delta = 1e-06,maxit=50,outer.maxit=50,verbose=F){
  if(missing(data.matrix)) stop("Argument data.matrix missing, with no default")
  II <- dim(data.matrix)[1] ##II instrad of I cause I is a fuction in R
  J <- dim(data.matrix)[2]
  cI <- II ##current I
  cJ <- J ##current J
  theta.outliers.old <- rep(F,II) ##ith entry will be true if theta_i is an outlier
  phi.outliers.old <- rep(F,J) ##jth entry will be true if phi_j is an outlier
  single.outliers.old <- matrix(F,II,J) ##ij entry will be true if y_is an outlier
  theta.outliers <- theta.outliers.old ##need this to now if change ocurred in outliers
  phi.outliers <- phi.outliers.old ##need this to know if chages occured in outlies
  single.outliers <- single.outliers.old
  flag1 <- NA ##these will be false if convergence not reacher,
  flag2 <- NA ## this will be false if outliers respectively cuase iter to stop
  if(remove.outliers){
    flag1 <- T;flag2<-T
    original.data.matrix <- data.matrix ##so we can get it back after outlier removal
    change.theta <- 1 #start with 1 
    change.phi <- 1
    change.single <- 1
    outer.iter <- 0 
    while(flag1 & flag2 & change.theta+change.phi+change.single >0 & outer.iter < outer.maxit) {
      outer.iter <- outer.iter + 1
      
      if((outer.iter%%3==0 & change.theta>0)  |
         (outer.iter%%3==1 & change.phi>0)){ #something has to change
        ##starting values
        phi <- apply(data.matrix, 2, mean)
        c <- sqrt(cJ/sum(phi[!phi.outliers]^2))
        phi <- c * phi
        theta <- (data.matrix[,!phi.outliers] %*% phi[!phi.outliers])/cJ
        iter <- 0
        change <- 1 #start with one
        theta.old <- rep(0, II)
        while(change > delta & iter < maxit) {
          iter <- iter + 1
          phi <- t(data.matrix[!theta.outliers,]) %*% theta[!theta.outliers] ##ignore the outliers
          c <- sqrt(cJ/sum(phi[!phi.outliers]^2))
          phi <- c * phi
          theta <- (data.matrix[,!phi.outliers] %*% phi[!phi.outliers])/cJ
          change <- max(abs(theta[!theta.outliers] - theta.old[!theta.outliers]))
          if(verbose) cat(paste("Outlier iteration:",outer.iter,"estimation iteration:",iter,"chage=",change,"\n"))
          theta.old <- theta
        }
        if(iter>=maxit){ ##convergence not reached. might as well get out
          warning(paste("No convergence in inner loop after",iter,"in outerler tieration",outer.iter,"\n"))
          flag1 <- F
        }
        if(mean(phi[!phi.outliers]<0)>.5){ ##for identifiability.. theta*phi = (-theta)*(-phi), i require that most phis are positive
          theta <- -theta
          phi <- -phi
        }
        theta <- as.vector(theta)
        phi <- as.vector(phi)
                
        data.matrixhat <- outer(theta, phi)
        resid <- data.matrix-data.matrixhat
      }
      ##DEALING WITH OUTLIERS
      ##we alternate removal of outliers
      ##if even iteration take out thetas that are outliers (as defined by Li and Wong).
      if(outer.iter%%3==1){ ## we start with single outliers
        single.outliers <- resid > large.threshold*quantile(abs(resid),normal.resid.quantile)
        single.outliers[apply(single.outliers,1,sum) > outlier.fraction*cJ,]<-rep(F,J)
        ##probably chip oulier, defer calling outlier
        single.outliers[,apply(single.outliers,2,sum) > outlier.fraction*cI]<-rep(F,II)
        ##probably probe outlier, defer calling outlier
        data.matrix[single.outliers] <- data.matrixhat[single.outliers]
        data.matrix[!single.outliers] <- original.data.matrix[!single.outliers]
        change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
        single.outliers.old <- single.outliers
      }
      else{
        sigma.theta <- sqrt(apply(resid[,!phi.outliers]^2, 1, sum)/(cJ - 1))
        sigma.phi <- sqrt(apply(resid[!theta.outliers,]^2, 2, sum)/(cI - 1))
        ###THETA OUTLIERS
        if(outer.iter%%3==2){
          theta.outliers <- sigma.theta > large.threshold*quantile(sigma.theta,normal.array.quantile) | theta^2/sum(theta^2) > large.variation
          cI <- sum(!theta.outliers)
          if(cI<3) {
            warning("No convergence achieved, too many outliers")
            flag2 <- F
          }
          ##single outliers in outlier chips are not longer single outliers
          single.outliers[theta.outliers,] <- rep(F,J)
          data.matrix[single.outliers] <- data.matrixhat[single.outliers]
          data.matrix[!single.outliers]<-original.data.matrix[!single.outliers]
          change.theta <- sum(abs(theta.outliers.old-theta.outliers)) #sum will be total of changes
          change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
          theta.outliers.old <- theta.outliers
        }
        ##PHI OUTLIERS
        else{
          phi.outliers <- sigma.phi > large.threshold*quantile(sigma.phi,normal.array.quantile) | phi^2/sum(phi^2) > large.variation | phi <0
          cJ <- sum(!phi.outliers)
          if(cJ<3) {
            warning("No convergence achieved, too many outliers")
            flag2 <- F
          }
          single.outliers[,phi.outliers] <- rep(F,II)
          data.matrix[single.outliers] <- data.matrixhat[single.outliers]
          data.matrix[!single.outliers]<-original.data.matrix[!single.outliers]
          change.phi <- sum(abs(phi.outliers.old-phi.outliers))
          change.single <- sum(abs(single.outliers.old-single.outliers)) #sum will be total of changes
          phi.outliers.old <- phi.outliers
        }
      }
      if(verbose){
        cat("chips used=",cI,", probes used=",cJ,", single outler=",sum(single.outliers),"\n")
        cat("Number of changes: single=",change.single,", theta=",change.theta,", phi=",change.phi,"\n",sep="")
      }
    }
    if(outer.iter>=outer.maxit){
      warning("No convergence achieved in outlier loop\n")
      flag2 <- F
    }
    all.outliers <- outer(theta.outliers,phi.outliers,FUN="|") | single.outliers
    sigma <- sqrt(sum(resid[!all.outliers]^2)/sum(!all.outliers))
    ##in case we leave iteration and these havent been defined
    sigma.theta <- sqrt(apply(resid[,!phi.outliers]^2, 1, sum)/(cJ - 1))
    sigma.phi <- sqrt(apply(resid[!theta.outliers,]^2, 2, sum)/(cI - 1))
  }
  ###code for NO OUTLIER REMOVAL
  else{
    flag1 <- T
    phi <- apply(data.matrix, 2, mean)
    c <- sqrt(J/sum(phi^2))
    phi <- c * phi
    theta <- (data.matrix %*% phi)/J

    iter <- 0
    change <- 1
    theta.old <- rep(0, II)
    while(change > delta & iter < maxit) {
      iter <- iter + 1
      phi <- t(data.matrix) %*% theta
      c <- sqrt(J/sum(phi^2))
      phi <- c * phi
      theta <- (data.matrix %*% phi)/J
      change <- max(abs(theta - theta.old))
      if(verbose) cat(paste("Iteration:",iter,"chage=",change,"\n"))
      theta.old <- theta
    }
    if(iter>=maxit){
      warning(paste("No convergence after",iter,"iterations.\n"))
      flag1 <- F
    }
    if(mean(phi[!phi.outliers]<0)>.5){
      ##for identifiability.. theta*phi = (-theta)*(-phi), i require that most phis are positive
      theta <- -theta
      phi <- -phi
    }
    theta <- as.vector(theta)
    phi <- as.vector(phi)
    data.matrixhat <- outer(theta, phi)
    sigma.theta <- sqrt(apply((data.matrix - data.matrixhat)^2, 1, sum)/(J - 1))
    sigma.phi <- sqrt(apply((data.matrix - data.matrixhat)^2, 2, sum)/(II - 1))
    sigma <- sqrt(sum((data.matrix - data.matrixhat)^2)/(II * J))
  }
  return(list(theta = theta, phi = phi, sigma.eps = sigma, sigma.theta = sigma.theta, sigma.phi=sigma.phi,theta.outliers=theta.outliers,phi.outliers=phi.outliers,single.outliers=single.outliers,convergence1=flag1,convergence2=flag2,iter = iter, delta = change)) 
}

affy.names <- function(x){
  if(missing(x)) stop("Argument x missing, with no default\n")
  unique(x$probe.names)
}

li.wong <- function(x,subset=NULL,ignore.mm=F,remove.outliers=T,
                    normal.array.quantile=0.5,
                    normal.resid.quantile=0.9,
                    large.threshold=3,
                    large.variation=0.8,
                    outlier.fraction=0.14,
                    delta = 1e-06,maxit=50,outer.maxit=50,verbose=F){
  if(missing(x)) stop("Argument x missing, with no default\n")
  if(ignore.mm)
    aux <- x$pm
  else
    aux <- x$pm-x$mm
  unique.probe.names <- unique(x$probe.names)
  if(!is.null(subset)) unique.probe.names <- unique.probe.names[subset]
  n <- length(unique.probe.names)
  theta <- matrix(0,n,x$nchips)
  sigma.theta <- matrix(0,n,x$nchips)
  outlier <- matrix(T,n,x$nchips)
  sigma.eps <- rep(0,n)
  convergence1 <- rep(T,n)
  convergence2 <- rep(T,n)
  for(i in 1:n){ ##faster than tapply?
    index <- x$probe.names==unique.probe.names[i]
    res <- fit.li.wong(t(aux[index,]),remove.outliers,normal.array.quantile,normal.resid.quantile,large.threshold,large.variation,outlier.fraction,delta,maxit,outer.maxit,verbose)
    theta[i,] <- res$theta
    sigma.theta[i,] <- res$sigma.theta
    sigma.eps[i] <- res$sigma.eps
    outlier[i,] <- res$theta.outliers
    convergence1[i] <- res$convergence1
    convergence2[i] <- res$convergence2
  }
  return(list(probe.name=unique.probe.names,theta=theta,sigma.theta=sigma.theta,sigma.eps=sigma.eps,outlier=outlier,convergence1=convergence1,convergence2=convergence2))
}

avg.diff <- function(x){
  if(missing(x)) stop("Argument x missing, with no default\n")
  unique.probe.names <- unique(x$probe.names)
  n <- length(unique.probe.names)
  avg.diff <- matrix(0,n,x$nchips)
  for(i in 1:n){ ##faster than tapply?
    index <- x$probe.names==unique.probe.names[i]
    avg.diff[i,] <- apply(x$pm[index,]-x$mm[index,],2,mean)
  }
  dimnames(avg.diff) <- list(unique.probe.names,x$chip.names)
  return(avg.diff)
}

affy.avg.diff <- function(x){
  if(missing(x)) stop("Argument x missing, with no default\n")
  unique.probe.names <- unique(x$probe.names)
  n <- length(unique.probe.names)
  avg.diff <- matrix(0,n,x$nchips)
  for(i in 1:n){ ##faster than tapply?
    index <- x$probe.names==unique.probe.names[i]
    aux <- x$pm[index,]-x$mm[index,]
    avg.diff[i,] <- apply(aux,2,function(x){
      o <- order(x)
      xx <- x[-c(o[1],o[length(x)])] #take out biggest and smallest
      mean(x[abs(x-mean(xx))<3*sd(xx)])
    })
  }
  dimnames(avg.diff) <- list(unique.probe.names,x$chip.names)
  return(avg.diff)
}

avg.log.ratio <- function(x){
  if(missing(x)) stop("Argument x missing, with no default\n")
  unique.probe.names <- unique(x$probe.names)
  n <- length(unique.probe.names)
  avg.log.ratio <- matrix(0,n,x$nchips)
  for(i in 1:n){ ##faster than tapply?
    index <- x$probe.name==unique.probe.names[i]
    avg.log.ratio[i,] <- apply(log2(x$pm[index,]/x$mm[index,]),2,mean)
  }
  dimnames(avg.log.ratio) <- list(unique.probe.names,x$chip.names)
  return(avg.log.ratio)
}

affy.mva <- function(x,xcol=1,ycol=1,pm=T,mm=T,subset=NULL,sample.size=10000,loess.it=F,span=1/3,return.it=F,plot.it=T,xlab="A",ylab="M",main="",pch=".",pm.col=3,mm.col=2,line.col=par()$col,lty=1,cex=1,family.loess="symmetric"){ 
  if(is.null(subset)) subset<-sample(1:x$nprob,sample.size)
  if(pm & !mm){
    aux1 <- x$pm[subset,xcol]
    aux2 <- x$pm[subset,ycol]
    if(plot.it) plot(sqrt(aux1*aux2),aux2/aux1,main=main,xlab=xlab,ylab=ylab,pch=pch,col=pm.col,log="xy",cex=cex)}
  if(mm & !pm){
    aux1 <- x$mm[subset,xcol]
    aux2 <- x$mm[subset,ycol]
    if(plot.it) plot(sqrt(aux1*aux2),aux2/aux1,main=main,xlab=xlab,ylab=ylab,pch=pch,col=mm.col,log="xy",cex=cex)}
  if(pm & mm){
    aux1 <- c(x$pm[subset,xcol],x$mm[subset,xcol])
    aux2 <- c(x$pm[subset,ycol],x$mm[subset,ycol])
    if(plot.it){
      plot(sqrt(aux1*aux2),aux2/aux1,main=main,xlab=xlab,ylab=ylab,type="n",log="xy")
      points(sqrt(aux1[(sample.size+1):(2*sample.size)]*
                  aux2[(sample.size+1):(2*sample.size)]),
             aux2[(sample.size+1):(2*sample.size)]/
             aux1[(sample.size+1):(2*sample.size)],
             col=mm.col,pch=pch,cex=cex)
      points(sqrt(aux1[1:sample.size]*aux2[1:sample.size]),
             aux2[1:sample.size]/aux1[1:sample.size],col=pm.col,pch=pch,cex=cex)
      
    }
  }
  if(plot.it) abline(h=1,lty=2)
  if(!pm & !mm) stop("At least one of pm or mm must be true.\n")
  if(loess.it | return.it) aux <- plot.mva(aux1,aux2,log.it=T,loess.it=T,span=span,return.it=T,plot.it=F,family.loess=family.loess)
  if(plot.it & loess.it){
    o <- order(sqrt(aux1*aux2))
    lines(sqrt(aux1*aux2)[o],exp(aux[o]),lty=lty,col=line.col)
  }
  if(return.it) return(aux)
  else return(NULL)
}
  


plot.mva<- function(x,y,log.it=F,loess.it=F,span=1/3,return.it=F,plot.it=T,xlab="A",ylab="M",main="",pch=".",col=1,line.col=2,lty=1,cex=1,family.loess="symmetric"){
  if(log.it){
    if(plot.it){
      plot(sqrt(x*y),y/x,main=main,xlab=xlab,ylab=ylab,pch=pch,col=col,cex=cex,log="xy")
    abline(h=1,lty=2)
    }
    if(loess.it){
      yy <- log(y)-log(x)
      xx <- (log(y)+log(x))/2
      aux <- loess(yy~xx,span=span,degree=1,family=family.loess)
      n <- length(xx)
      if(plot.it){
      o <- order(xx)
      yy <- yy[o]
      xx <- xx[o]
      fitted <- aux$fitted[o]
      if(n>1000){
          o <- round(seq(1,n,len=1000))##if to many points only use 1000
          lines(exp(xx[o]),exp(fitted[o]),lty=lty,col=line.col)
        }
        else{
          lines(exp(xx),exp(fitted),lty=lty,col=line.col)
        }
      }
    }
    if(return.it) return(aux$fitted)
  }
  else{
    if(plot.it){
      plot((x+y)/2,y-x,main=main,xlab=xlab,ylab=ylab,pch=pch,col=col,cex=cex)
      abline(h=0,lty=2)
    }
    if(loess.it){
      yy <- y-x
      xx <- (y+x)/2
      aux <- loess(yy~xx,span=span,degree=1,family=family.loess)
      n <- length(xx)
      if(plot.it){
        o <- order(xx)
        yy <- yy[o]
        xx <- xx[o]
        fitted <- aux$fitted[o]
        if(n>1000){
          o <- round(seq(1,n,len=1000))##if to many points only use 1000
          lines(xx[o],fitted[o],lty=lty,col=line.col)
        }
        else{
          lines(xx,fitted,lty=lty,col=line.col)
        }
      }
    }
    if(return.it) return(aux$fitted)
  }
}

exp.comp <- function(x,verbose=F){
  unique.probe.names <- unique(x$probe.names)
  n <- length(unique.probe.names)
  res <- array(0,dim=c(8,n,x$nchips))
  dimnames(res) <- list(c("avg.diff","li.wong","avg.log.ratio","efron.et.al",
                          "2nd.largest.ratio",
                          "avg.log.pm","2nd.largest.pm","li.wong.no.mm"),
                        unique.probe.names,x$chip.names)
  for(i in 1:n){ ##faster than tapply?
    if(verbose) cat(i,"out of",n,": ",unique.probe.names[i],"\n")
    index <- x$probe.names==unique.probe.names[i]
    pms <- x$pm[index,]
    mms <- x$mm[index,]
    lpms <- log2(pms)
    lmms <- log2(mms)
    index <- x$probe.names==unique.probe.names[i]
    index <- x$probe.names==unique.probe.names[i]
    res[1,i,] <- apply(pms-mms,2,mean)
    res[2,i,] <- fit.li.wong(t(pms-mms),remove.outliers=F,maxit=150)$theta
    res[3,i,] <- apply(lpms-lmms,2,mean)
    res[4,i,] <- apply(lpms-lmms/2,2,mean)
    res[5,i,] <- apply(lpms-lmms,2,function(x) x[order(-x)[2]])
    res[6,i,] <- apply(lpms,2,mean)
    res[7,i,] <- apply(lpms,2,function(x) x[order(-x)[2]])
    res[8,i,] <- fit.li.wong(t(pms),remove.outlier=F,maxit=150)$theta
  }
   return(res)
}

affy.subset <- function(x,probes=NULL,chips=NULL,genes=NULL){
  if(is.null(chips) & is.null(probes) & is.null(genes)) return(x)
  else{
    unique.gene.names <- unique(x$probe.names)
    if(is.null(chips)) chips <- 1:x$nchips
    if(is.null(genes)){
      flag <- F ##to avoid intersect
      genes <- 1:x$nprob
    }
    else{
      aux <- c()
      for(i in unique.gene.names[genes]){ ##not always the same so cant use
        ##sapply!
        aux <- c(aux,(1:x$nprob)[x$probe.name==i])
      }        
      genes <- aux
      flag <- T
    }
    if(is.null(probes)) probes <- genes
    else{
      if(flag){
        probes <- intersect(genes,1:x$nprob)
      }
    }        
    return(pm=x$pm[probes,chips],mm=x$mm[probes,chips],
           probe.names=x$probe.names[probes],
           x=x$x[probes],y=y$probes,ncol=x$ncol,nrow=y$nrow,
           chip.names=x$chip.names[chips])
  }
}

mva.pairs <- function(x,labels=colnames(x),log.it=T,span=1/3,family.loess="symmetric",digits=3,line.col=2){
  if(log.it) x <-log2(x)
  J <- dim(x)[2]
  frame()
  par(mfrow=c(J,J),mgp=c(0,.2,0),mar=c(1,1,1,1),oma=c(1,1.4,1,1))
  for(j in 1:(J-1)){
    par(mfg=c(j,j));plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="");text(1,1,labels[j],cex=2)
    for(k in (j+1):J){
      par(mfg=c(j,k))
      yy <- x[,j]-x[,k]
      xx <-(x[,j]+x[,k])/2
      xx <- xx
      yy <- yy
      aux <- loess(yy~xx,degree=1,span=span,family=family.loess)$fitted
      plot(xx,yy,pch=".",xlab="",ylab="",tck=0)
      o <- order(xx)
      lines(xx[o],aux[o],col=line.col)
      par(mfg=c(k,j))
      sigma <- sd(yy)
      txt <- format(c(sigma,0.123456789),digits=digits)
      plot(c(0,1),c(0,1),type="n",ylab="",xlab="",xaxt="n",yaxt="n")
      text(0.5,0.5,txt,cex=2)
    }
  }
  par(mfg=c(J,J));plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="");
  text(1,1,labels[J],cex=2)
  mtext("A",1,outer=T,cex=1.5)
  mtext("M",2,outer=T,cex=1.5,las=1)

}

mva.pairs.sr <- function(x,axis.range,labels=colnames(x),log.it=T,span=1/3,family.loess="symmetric",digits=3,line.col=2){
  if(log.it) x <-log2(x)
  frame()
  J <- dim(x)[2]
  par(mfrow=c(J,J),mgp=c(0,.2,0),mar=c(1,1,1,1),oma=c(1,1.4,1,1))
  for(j in 1:(J-1)){
    par(mfg=c(j,j));plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="");text(1,1,labels[j],cex=2)
    for(k in (j+1):J){
      par(mfg=c(j,k))
      yy <- x[,j]-x[,k]
      xx <-(x[,j]+x[,k])/2
      xx <- xx
      yy <- yy
      aux <- loess(yy~xx,degree=1,span=span,family=family.loess)$fitted
      plot(xx,yy,pch=".",xlab="",ylab="",tck=0,ylim=axis.range,xlim=axis.range)
      o <- order(xx)
      lines(xx[o],aux[o],col=line.col)
      par(mfg=c(k,j))
      sigma <- sd(yy)
      txt <- format(c(sigma,0.123456789),digits=digits)
      plot(c(0,1),c(0,1),type="n",ylab="",xlab="",xaxt="n",yaxt="n")
      text(0.5,0.5,txt,cex=2)
    }
  }
  par(mfg=c(J,J));plot(1,1,type="n",xaxt="n",yaxt="n",xlab="",ylab="");
  text(1,1,labels[J],cex=2)
  mtext("A",1,outer=T,cex=1.5)
  mtext("M",2,outer=T,cex=1.5,las=1)

}

normalize <- function(mat,sample.size=min(round(dim(mat)[1]/10),10000),
                      epsilon=10^-5,maxit=100,log.it=T,verbose=F,span=1/3,
                      family.loess="symmetric"){
  J <- dim(mat)[2]
  II <- dim(mat)[1]
  if(sample.size>II) sample.size <- II

  index <- sample(1:II,sample.size)
  oldData <- mat
  newData <- mat
  if(log.it){
    oldData <- log2(oldData)
    newData <- log2(newData)
  }
  change <- epsilon +1
  fs <- matrix(0,dim(oldData)[1],dim(oldData)[2])##contains what we substract
  means <- array(0,dim=c(J,J,dim(oldData)[1]))
  iter <- 0
  while(change > epsilon & iter < maxit){
    iter <- iter+1
    for(j in 1:(J-1)){
      for(k in (j+1):J){
        y <- newData[,j]-newData[,k]
        x <-(newData[,j]+newData[,k])/2
        xx <- x[index]
        yy <- y[index]
        #aux <- lowess(xx,yy,f=span)$y#fitted
        aux <-loess(yy~xx,span=span,degree=1,family=family.loess,
                    surface="direct")
        means[j,k,] <- predict.loess(aux,data.frame(xx=x))
        means[k,j,] <- -means[j,k,]
        if(verbose) cat("Done with",j,"vs",k," in iteration ",iter,"\n")
      }
    }
    addtofs <- sapply(1:J,function(j) apply(means[j,-j,],2,sum)/J )
    fs <- fs+addtofs
    newData <- oldData-fs
    change <- max(apply((addtofs[index,])^2,2,mean))
    if(verbose) cat(iter,change,"\n")
    oldfs <- fs
  }
  if(maxit<=iter) warning(paste("No convergence after",maxit,"iterations.\n"))
  return(fitted=fs,residuals=newData)
}


ratio.hist <- function(x,j,prob=c(0,.25,.75,.95,1),
                       nclass=75){
  lpm <- log2(x$pm[,j])
  lmm <- log2(x$mm[,j])
  auxx <- quantile(lpm+lmm,prob)
  for(i in 1:length(auxx)){
    hist((lpm-lmm)[(lpm+lmm)>=auxx[i] & (lpm+lmm)<=auxx[i+1]],nclass=nclass,main=aux[i],xlab="",xlim=range(lpm-lmm),main=paste(round(prob[i]*100),"% - ",round(prob[i+1]*100),sep=""))
  }
}

probe.profile <- function(x,probe.set,chips=1:x$nchips,by.chip=T,pm.col="green",mm.col="red",global.ylim=T){
  index <- (1:x$nprob)[x$probe.name==probe.set]
  probes <- x$probe.number[index]
  o <- order(probes)
  index <- index[o]
  probes <- probes[o]
  GYLIM <- log2(range(rbind(x$pm[index,chips],x$mm[index,chips])))
  if(by.chip){
    for(i in chips){
      p <- log2(x$pm[index,i])
      m <- log2(x$mm[index,i])
      if(global.ylim) YLIM <- GYLIM else YLIM <- range(c(p,m))
      plot(probes,p,type="n",ylim=YLIM,
           xlab="probe number",ylab="log intensity",
           main=x$chip.names[i])
      points(probes,p,pch="p",col=pm.col,type="b")
      points(probes,m,pch="m",col=mm.col,type="b")
    }
  }
  else{
    for(i in 1:length(index)){
      p <- log2(x$pm[index[i],chips])
      m <- log2(x$mm[index[i],chips])
      if(global.ylim) YLIM <- GYLIM else YLIM <- range(c(p,m))
      plot(chips,p,type="n",ylim=YLIM,
           xlab="chip",ylab="log intensity",
           main=paste("probe",as.character(probes[i])),xaxt="n")
      axis(1,at=chips,labels=x$chip.names)
      points(chips,p,pch="p",col=pm.col,type="b")
      points(chips,m,pch="m",col=mm.col,type="b")
    }
  }
}
   
hlog <- function(x,constant=2.7){ #constant is where the change occurs
  if(constant<=0){
    warning("constant less than or equal to 0. Returning log(x)\n")
    return(log(x))
  }
  else{
    if(constant==Inf)
      return(x)
    else{
      aux <- (abs(x)<constant)*(x) +
        (abs(x)>=constant)*(sign(x)*(constant*log(abs(x/constant))+constant))
      aux[x==0] <- 0
      return(aux)
    }
  }
}


